@@ -7,6 +7,7 @@
 #include "animation.h"
 
 #ifdef HAVE_ANIMATION
+#include <ge-support.h>
 #include <glib/gtimer.h>
 
 struct _AnimationInfo {
@@ -32,10 +33,10 @@ static int         animation_timer_id = 0;
 static gboolean animation_timeout_handler (gpointer data);
 
 /* This forces a redraw on a widget */
-static gboolean
+static void
 force_widget_redraw (GtkWidget *widget)
 {
-	if (GTK_IS_PROGRESS_BAR (widget))
+	if (IS_PROGRESS_BAR (widget))
 		gtk_widget_queue_resize (widget);
 	else
 		gtk_widget_queue_draw (widget);
@@ -141,7 +142,6 @@ update_animation_info (gpointer key, gpointer value, gpointer user_data)
 {
 	AnimationInfo *animation_info = value;
 	GtkWidget *widget = key;
-	gdouble elapsed;
 	
 	if ((widget == NULL) || (animation_info == NULL))
 		g_assert_not_reached ();
@@ -150,7 +150,7 @@ update_animation_info (gpointer key, gpointer value, gpointer user_data)
 	if (!GTK_WIDGET_DRAWABLE (widget))
 		return TRUE;
 	
-	if (GTK_IS_PROGRESS_BAR (widget))
+	if (IS_PROGRESS_BAR (widget))
 	{
 		gfloat fraction = gtk_progress_bar_get_fraction (GTK_PROGRESS_BAR (widget));
 		
@@ -260,7 +260,7 @@ clearlooks_animation_progressbar_add (GtkWidget *progressbar)
 void
 clearlooks_animation_connect_checkbox (GtkWidget *widget)
 {
-	if (GTK_IS_CHECK_BUTTON (widget))
+	if (IS_CHECK_BUTTON (widget))
 	{
 		if (!g_slist_find_custom (connected_widgets, widget, find_signal_info))
 		{
