@@ -19,6 +19,7 @@
  * Written by Owen Taylor <otaylor@redhat.com>
  * and by Alexander Larsson <alexl@redhat.com>
  * Modified by Richard Stellingwerff <remenic@gmail.com>
+ * Modified by Kulyk Nazar <schamane@myeburg.net>
  */
 
 #include <gtk/gtkrc.h>
@@ -39,14 +40,16 @@ struct _ClearlooksRcStyle
 {
   GtkRcStyle parent_instance;
 
-  GdkColor spot_color;
-  gboolean has_spot_color;
+  GdkColor scrollbar_color;
+  gboolean has_scrollbar_color;
   double contrast;
   guint8 sunkenmenubar;
   guint8 progressbarstyle;
   guint8 menubarstyle;
   guint8 menuitemstyle;
   guint8 listviewitemstyle;
+  guint8 scrollbarstyle;
+  gboolean animation;
 };
 
 struct _ClearlooksRcStyleClass
