@@ -1,7 +1,5 @@
 #include "crux-draw.h"
 #include "crux-style.h"
-#include "crux-rc-style.h"
-#include "crux-common.h"
 
 #include <ge-support.h>
 
@@ -15,6 +13,9 @@
 
 #define OUTLINE_GRAY 46/255.0, 52/255.0, 54/255.0
 
+#define CRUX_LIGHT(xx, yy) yy.r = xx.r + 56/255.0;yy.g = xx.g + 47/255.0;yy.b = xx.b + 45/255.0;
+#define CRUX_DARK(xx, yy) yy.r = xx.r - 25/255.0;yy.g = xx.g - 27/255.0;yy.b = xx.b - 21/255.0;
+
 GtkStyleClass *parent_style_class;
 
 static void
@@ -51,15 +52,6 @@ draw_box (GtkStyle *style,
 	  GtkWidget *widget,
 	  const gchar *detail, gint x, gint y, gint width, gint height);
 
-static void
-draw_polygon (GtkStyle *style,
-	      GdkWindow *window,
-	      GtkStateType state_type,
-	      GtkShadowType shadow_type,
-	      GdkRectangle *area,
-	      GtkWidget *widget,
-	      const gchar *detail, GdkPoint *points, gint npoints, gint fill);
-
 static void
 draw_arrow (GtkStyle *style,
 	    GdkWindow *window,
@@ -71,33 +63,6 @@ draw_arrow (GtkStyle *style,
 	    GtkArrowType arrow_type,
 	    gint fill, gint x, gint y, gint width, gint height);
 
-static void
-draw_diamond (GtkStyle *style,
-	      GdkWindow *window,
-	      GtkStateType state_type,
-	      GtkShadowType shadow_type,
-	      GdkRectangle *area,
-	      GtkWidget *widget,
-	      const gchar *detail, gint x, gint y, gint width, gint height);
-
-static void
-draw_string (GtkStyle *style,
-	     GdkWindow *window,
-	     GtkStateType state_type,
-	     GdkRectangle *area,
-	     GtkWidget *widget,
-	     const gchar *detail, gint x, gint y, const gchar *string);
-#if 0
-static void
-draw_flat_box (GtkStyle *style,
-	       GdkWindow *window,
-	       GtkStateType state_type,
-	       GtkShadowType shadow_type,
-	       GdkRectangle *area,
-	       GtkWidget *widget,
-	       const gchar *detail, gint x, gint y, gint width, gint height);
-#endif
-
 static void
 draw_check (GtkStyle *style,
 	    GdkWindow *window,
@@ -247,25 +212,6 @@ paint_menuitem (cairo_t *cr, GtkStyle *style, GtkStateType state_type,
 	cairo_pattern_destroy (crp);
 }
 
-static void
-paint_background_area (GtkStyle *style,
-		       GdkWindow *window,
-		       GtkStateType state_type,
-		       GdkRectangle *area,
-		       int x, int y, int width, int height)
-{
-    GdkGC *gc = style->bg_gc[GTK_STATE_NORMAL];
-
-    if (area != 0)
-	gdk_gc_set_clip_rectangle (gc, area);
-
-    gdk_draw_rectangle (window, gc, TRUE, x, y, width, height);
-
-    if (area != 0)
-	gdk_gc_set_clip_rectangle (gc, NULL);
-}
-
-
 static void
 paint_button (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkShadowType shadow_type,
 	gdouble x, gdouble y, gdouble width, gdouble height)
@@ -403,7 +349,6 @@ paint_shadow (cairo_t *cr, GtkStyle *style,
 	cairo_rectangle (cr, x, y, width, height);
 	if (shadow_type == GTK_SHADOW_OUT || shadow_type == GTK_SHADOW_ETCHED_OUT)
 	{
-		CairoColor c1, c2;
 		cairo_set_source_rgb (cr, OUTLINE_GRAY);
 		cairo_stroke (cr);
 	}
@@ -606,8 +551,8 @@ paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkPr
 	cairo_rectangle (cr, x, y, width, height);
 
 	ge_gdk_color_to_cairo (&style->base[GTK_STATE_SELECTED], &c1);
-	ge_shade_color (&c1, 0.8, &c2); /* darken */
-	ge_shade_color (&c1, 1.4, &c1); /* lighten */
+	CRUX_DARK(c1, c2)
+	CRUX_LIGHT(c1, c1)
 	if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT || orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
 		crp = cairo_pattern_create_linear (x, y, x, y + height);
 	else
@@ -688,77 +633,6 @@ paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkPr
 	}
 }
 
-static void
-paint_arrow (GdkWindow *window, GdkGC *gc, GtkArrowType arrow_type,
-	     int x, int y, int width, int height)
-{
-    int half_width, half_height;
-    int center_x, center_y;
-
-    if ((width & 1) == 0)
-	width = width - 1;
-    if ((height & 1) == 0)
-	height = height - 1;
-
-    half_width = width / 2;
-    half_height = height / 2;
-
-    center_x = x + half_width;
-    center_y = y + half_height;
-
-    switch (arrow_type)
-    {
-	int i;
-	static int offset[5] = { 0, -1, -2, -3, -4 };
-	static int length[5] = { 0, 2, 4, 6, 8 };
-	static const int size = 4;
-
-    case GTK_ARROW_UP:
-	for (i = 0; i < size; i++)
-	{
-	    gdk_draw_line (window, gc,
-			   center_x + offset[i],
-			   center_y - 2 + i,
-			   center_x + offset[i] + length[i],
-			   center_y - 2 + i);
-	}
-	break;
-
-    case GTK_ARROW_DOWN:
-	for (i = 0; i < size; i++)
-	{
-	    gdk_draw_line (window, gc,
-			   center_x + offset[i],
-			   center_y + 2 - i,
-			   center_x + offset[i] + length[i],
-			   center_y + 2 - i);
-	}
-	break;
-
-    case GTK_ARROW_LEFT:
-	for (i = 0; i < size; i++)
-	{
-	    gdk_draw_line (window, gc,
-			   center_x - 2 + i,
-			   center_y + offset[i],
-			   center_x - 2 + i,
-			   center_y + offset[i] + length[i]);
-	}
-	break;
-
-    case GTK_ARROW_RIGHT:
-	for (i = 0; i < size; i++)
-	{
-	    gdk_draw_line (window, gc,
-			   center_x + 2 - i,
-			   center_y + offset[i],
-			   center_x + 2 - i,
-			   center_y + offset[i] + length[i]);
-	}
-	break;
-    }
-}
-
 /* style functions */
 
 static void
@@ -885,10 +759,20 @@ draw_shadow (GtkStyle *style,
 	if (widget && (GTK_IS_COMBO (widget->parent) || GTK_IS_COMBO_BOX_ENTRY (widget->parent)))
 	{
 		GtkWidget *button;
-		GValue *value;
-		width += 2;
+		if (ge_widget_is_ltr (widget))
+			width += 2;
+		else
+		{
+			x -= 3;
+			width += 3;
+		}
+
 		if (area == NULL)
 			area = &area2;
+		g_object_set_data ((GObject*) widget->parent, "entry", widget);
+		button = g_object_get_data ((GObject*) widget->parent, "button");
+		if (GTK_IS_WIDGET (button))
+			gtk_widget_queue_draw_area (button,  button->allocation.x, button->allocation.y, button->allocation.width,button->allocation.height);
 	}
 	cr = ge_gdk_drawable_to_cairo (window, area);
 
@@ -915,28 +799,46 @@ draw_box (GtkStyle *style,
 	CHECK_ARGS
 	SANITIZE_SIZE
 
-    if (DETAIL ("spinbutton")) {
-      if (y > (height / 3)) {
-	/* this is a down arrow shadow */
-	y += 2;
-      }
-    }
-
     debug ("draw_box: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, x, y, width, height);
 
+	if (DETAIL ("spinbutton"))
+	{
+		/* this is a border round spin buttons - we don't want anything here */
+		return;
+	}
+
 	cr = ge_gdk_drawable_to_cairo (window, area);
 
 	/* no pressed state for scollbar buttons yet... */
 	if (DETAIL ("vscrollbar") || DETAIL ("hscrollbar"))
 		shadow_type = GTK_SHADOW_OUT;
 
-	if (DETAIL ("button") || DETAIL ("optionmenu"))
+	if (DETAIL ("button") || DETAIL ("optionmenu") || DETAIL ("spinbutton_down") || DETAIL ("spinbutton_up"))
 	{
+		if (DETAIL ("spinbutton_down") || DETAIL ("spinbutton_up"))
+		{
+			/* add some padding */
+			x++; y++; width -= 2;
+			if (DETAIL ("spinbutton_up")) 
+				height--;
+			else
+				height -= 2;
+		}
+		
 		if (widget && (GTK_IS_COMBO (widget->parent) || GTK_IS_COMBO_BOX_ENTRY (widget->parent)))
 		{
 			/* Combobox buttons */
+			gboolean focused = FALSE;
+			GtkWidget *entry;
 			/* TODO: need RTL support */
+			if ((entry = g_object_get_data ((GObject*) widget->parent, "entry")))
+			{
+				focused = (GTK_WIDGET_HAS_FOCUS (entry));
+				state_type = GTK_WIDGET_STATE (entry);
+			}
+
+
 			if (state_type == GTK_STATE_INSENSITIVE)
 				gdk_cairo_set_source_color (cr, &style->base[GTK_STATE_INSENSITIVE]);
 			else
@@ -944,7 +846,14 @@ draw_box (GtkStyle *style,
 			cairo_rectangle (cr, x, y, width, height);
 			cairo_fill (cr);
 
-			paint_entry_shadow (cr, style, state_type, FALSE, x - 4, y, width + 4, height);
+			g_object_set_data ((GObject*) widget->parent, "button", widget);
+
+			if (ge_widget_is_ltr (widget))
+				paint_entry_shadow (cr, style, state_type, focused, x - 4, y, width + 4, height);
+			else
+				paint_entry_shadow (cr, style, state_type, focused, x, y, width + 4, height);
+
+
 			x += 3; y += 3;
 			width -= 6; height -= 6;
 		}
@@ -966,7 +875,7 @@ draw_box (GtkStyle *style,
 	else if (DETAIL ("buttondefault"))
 	{
 		/* draw Default Button marking, but only when sensitive */
-		if (GTK_WIDGET_SENSITIVE (widget))
+		if (widget && GTK_WIDGET_SENSITIVE (widget))
 		{
 			gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_SELECTED]);
 			cx = x + 0.5; cy = y + 0.5; cw = width - 1.0; ch = height - 1.0;
@@ -1011,11 +920,11 @@ draw_box (GtkStyle *style,
 	{
 		if (GTK_IS_VSCALE (widget))
 		{
-			x += 2; width -= 4;
+			x += 1; width -= 2;
 		}
 		else
 		{
-			y += 2; height -= 4;
+			y += 1; height -= 2;
 		}
 		paint_button (cr, style, GTK_STATE_NORMAL, GTK_SHADOW_OUT, x, y, width, height);
 	}
@@ -1031,17 +940,20 @@ draw_box (GtkStyle *style,
 		/* fill  */
 		cairo_rectangle (cr, x, y, width, height);
 
-		if (DETAIL ("toolbar") || DETAIL ("menubar") || DETAIL ("vscrollbar") || DETAIL ("hscrollbar"))
+		if (DETAIL ("toolbar") || DETAIL ("menubar") || DETAIL ("vscrollbar") || DETAIL ("hscrollbar") || DETAIL ("handlebox") || DETAIL ("dockitem"))
 		{
 			if (shadow_type == GTK_SHADOW_OUT || shadow_type == GTK_SHADOW_ETCHED_OUT)
 			{
 				cairo_pattern_t *crp;
 				CairoColor c1, c2;
+				gdouble shade_v = 0.1;
 
 				crp = cairo_pattern_create_linear (x, y, x, y + height);
 				ge_gdk_color_to_cairo (&style->bg[state_type], &c1);
-				ge_shade_color (&c1, 0.9, &c2);
-				ge_shade_color (&c1, 1.1, &c1);
+				if (DETAIL ("vscrollbar") || DETAIL ("hscrollbar")) 
+					shade_v = 0.3;
+				ge_shade_color (&c1, 1.0 - shade_v, &c2);
+				ge_shade_color (&c1, 1.0 + shade_v, &c1);
 				/*
 				cairo_pattern_add_color_stop_rgb (crp, 0.0, 238/255.0, 238/255.0, 236/255.0);
 				cairo_pattern_add_color_stop_rgb (crp, 1.0, 185/255.0, 189/255.0, 182/255.0);
@@ -1071,153 +983,6 @@ draw_box (GtkStyle *style,
 
 }
 
-#if 0
-static void
-draw_polygon (GtkStyle *style,
-	      GdkWindow *window,
-	      GtkStateType state_type,
-	      GtkShadowType shadow_type,
-	      GdkRectangle *area,
-	      GtkWidget *widget,
-	      const gchar *detail, GdkPoint *points, gint npoints, gint fill)
-{
-#ifndef M_PI
-#define M_PI    3.14159265358979323846
-#endif /*
-        * M_PI
-        */
-#ifndef M_PI_4
-#define M_PI_4  0.78539816339744830962
-#endif /*
-        * M_PI_4
-        */
-
-    static const gdouble pi_over_4 = M_PI_4;
-    static const gdouble pi_3_over_4 = M_PI_4 * 3;
-
-    GdkGC *gc1;
-    GdkGC *gc2;
-    GdkGC *gc3;
-    GdkGC *gc4;
-    gdouble angle;
-    gint xadjust;
-    gint yadjust;
-    gint i;
-
-    CHECK_ARGS
-    g_return_if_fail (points != NULL);
-
-    debug ("draw_polygon: detail=%s state=%d shadow=%d\n",
-	    detail, state_type, shadow_type);
-
-    switch (shadow_type)
-    {
-    case GTK_SHADOW_IN:
-	gc1 = style->light_gc[state_type];
-	gc2 = style->dark_gc[state_type];
-	gc3 = style->light_gc[state_type];
-	gc4 = style->dark_gc[state_type];
-	break;
-    case GTK_SHADOW_ETCHED_IN:
-	gc1 = style->light_gc[state_type];
-	gc2 = style->dark_gc[state_type];
-	gc3 = style->dark_gc[state_type];
-	gc4 = style->light_gc[state_type];
-	break;
-    case GTK_SHADOW_OUT:
-	gc1 = style->dark_gc[state_type];
-	gc2 = style->light_gc[state_type];
-	gc3 = style->dark_gc[state_type];
-	gc4 = style->light_gc[state_type];
-	break;
-    case GTK_SHADOW_ETCHED_OUT:
-	gc1 = style->dark_gc[state_type];
-	gc2 = style->light_gc[state_type];
-	gc3 = style->light_gc[state_type];
-	gc4 = style->dark_gc[state_type];
-	break;
-    default:
-	return;
-    }
-
-    if (area)
-    {
-	gdk_gc_set_clip_rectangle (gc1, area);
-	gdk_gc_set_clip_rectangle (gc2, area);
-	gdk_gc_set_clip_rectangle (gc3, area);
-	gdk_gc_set_clip_rectangle (gc4, area);
-    }
-
-    if (fill)
-	gdk_draw_polygon (window, style->bg_gc[state_type], TRUE, points,
-			  npoints);
-
-    npoints--;
-
-    for (i = 0; i < npoints; i++)
-    {
-	if ((points[i].x == points[i + 1].x) &&
-	    (points[i].y == points[i + 1].y))
-	{
-	    angle = 0;
-	}
-	else
-	{
-	    angle = atan2 (points[i + 1].y - points[i].y,
-			   points[i + 1].x - points[i].x);
-	}
-
-	if ((angle > -pi_3_over_4) && (angle < pi_over_4))
-	{
-	    if (angle > -pi_over_4)
-	    {
-		xadjust = 0;
-		yadjust = 1;
-	    }
-	    else
-	    {
-		xadjust = 1;
-		yadjust = 0;
-	    }
-
-	    gdk_draw_line (window, gc1,
-			   points[i].x - xadjust, points[i].y - yadjust,
-			   points[i + 1].x - xadjust,
-			   points[i + 1].y - yadjust);
-	    gdk_draw_line (window, gc3, points[i].x, points[i].y,
-			   points[i + 1].x, points[i + 1].y);
-	}
-	else
-	{
-	    if ((angle < -pi_3_over_4) || (angle > pi_3_over_4))
-	    {
-		xadjust = 0;
-		yadjust = 1;
-	    }
-	    else
-	    {
-		xadjust = 1;
-		yadjust = 0;
-	    }
-
-	    gdk_draw_line (window, gc4,
-			   points[i].x + xadjust, points[i].y + yadjust,
-			   points[i + 1].x + xadjust,
-			   points[i + 1].y + yadjust);
-	    gdk_draw_line (window, gc2, points[i].x, points[i].y,
-			   points[i + 1].x, points[i + 1].y);
-	}
-    }
-    if (area)
-    {
-	gdk_gc_set_clip_rectangle (gc1, NULL);
-	gdk_gc_set_clip_rectangle (gc2, NULL);
-	gdk_gc_set_clip_rectangle (gc3, NULL);
-	gdk_gc_set_clip_rectangle (gc4, NULL);
-    }
-}
-#endif
-
 static void
 draw_arrow (GtkStyle *style,
 	    GdkWindow *window,
@@ -1229,308 +994,105 @@ draw_arrow (GtkStyle *style,
 	    GtkArrowType arrow_type,
 	    gint fill, gint x, gint y, gint width, gint height)
 {
-    eazel_theme_data *theme_data;
+	cairo_t *cr;
+	gboolean is_combo_box_entry;
 
-    /* FIXME GNOME2: bad hack added to make arrows draw large enough */
-    width = width + 8;
-    height = height + 6;
-    x = x - 4;
-    y = y - 3;
-
-    CHECK_ARGS
-    /*SANITIZE_SIZE*/
-
-    theme_data = CRUX_RC_STYLE (style->rc_style)->theme_data;
-    g_assert (theme_data != NULL);
-
-    debug ("draw_arrow: detail=%s state=%d shadow=%d arrow_type=%d x=%d y=%d w=%d h=%d\n",
+	CHECK_ARGS;
+	debug ("draw_arrow: detail=%s state=%d shadow=%d arrow_type=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, arrow_type, x, y, width, height);
 
-    if (DETAIL ("vscrollbar") || DETAIL ("hscrollbar"))
-    {
-	x++;y++; width--; height--;
-	    /*
-	int type = 0;
-	switch (arrow_type)
+	if (DETAIL ("vscrollbar") || DETAIL ("hscrollbar"))
 	{
-	case GTK_ARROW_UP:
-	    type = EAZEL_ENGINE_ARROW_UP;
-	    break;
+		/* add some padding */
+		x++; y++; height--; width--; 
+		if (DETAIL ("hscrollbar"))
+			width--;
+		else
+			height--;
 
-	case GTK_ARROW_DOWN:
-	    type = EAZEL_ENGINE_ARROW_DOWN;
-	    break;
-
-	case GTK_ARROW_LEFT:
-	    type = EAZEL_ENGINE_ARROW_LEFT;
-	    break;
-
-	case GTK_ARROW_RIGHT:
-	    type = EAZEL_ENGINE_ARROW_RIGHT;
-	    break;
+		/* random adjustments... */
+		if (arrow_type == GTK_ARROW_DOWN)
+			y++;
+		else if (arrow_type == GTK_ARROW_RIGHT)
+			x++;
+			
 	}
-	*/
-	}
-    if (DETAIL ("spinbutton"))
-    {
-	int window_width, window_height;
-	int tem_x, tem_y;
-
-	if (widget != 0 && !GTK_WIDGET_IS_SENSITIVE (widget))
-	    state_type = GTK_STATE_INSENSITIVE;
-
-	gdk_window_get_size (window, &window_width, &window_height);
-
-	if (state_type != GTK_STATE_INSENSITIVE)
+	else if (DETAIL ("spinbutton"))
 	{
-	    draw_box (style, window, state_type, shadow_type,
-		      area, widget, detail,
-		      x, y - (arrow_type == GTK_ARROW_DOWN),
-		      width, height + 1);
+		if (arrow_type == GTK_ARROW_DOWN)
+			y+=2;
+		else if (arrow_type == GTK_ARROW_UP)
+			y--;
+
+		x++; width--;
+
 	}
-
-	/*
-	  else if (arrow_type == GTK_ARROW_UP)
-	  {
-	  XXX A hack, assumes that up arrow is drawn before
-	  XXX down arrow. (Currently it is)
-
-	  draw_shadow (style, window, GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-	  NULL, widget, "entry", x - 2, 0,
-	  width + 4, window_height);
-	  }
-	*/
-
-	tem_x = x + (width / 2);
-	tem_y = y + (height / 2);
-	if (arrow_type == GTK_ARROW_UP)
+	else if (DETAIL ("menuitem"))
 	{
-	    int i;
-	    tem_y--;
-	    for (i = 0; i < 4; i++)
-	    {
-		gdk_draw_line (window, style->fg_gc[state_type],
-			       tem_x - i, tem_y + i,
-			       tem_x + i, tem_y + i);
-	    }
+		/* make slightly smaller... */
+		x++;
+		height-=2;
+	} else if (ge_is_combo_box_entry (widget) || ge_is_combo (widget) || ge_is_combo_box (widget, FALSE))
+	{
+		x+=2; y+=3; width -= 4; height -= 4;
+		if (ge_is_combo_box (widget, FALSE))
+			x++;
+		if (ge_is_combo_box_entry (widget))
+			x--;
 	}
 	else
 	{
-	    int i;
-	    tem_y -= 2;
-	    for (i = 0; i < 4; i++)
-	    {
-		gdk_draw_line (window, style->fg_gc[state_type],
-			       tem_x - i, tem_y + (4 - i),
-			       tem_x + i, tem_y + (4 - i));
-	    }
+		/* catch all to add some padding... */
+		x++;
+		y++;
+		width -= 2;
+		height -= 2;
 	}
-	if (state_type != GTK_STATE_INSENSITIVE)
+
+	/* make triangle equilateral */
+	if (arrow_type == GTK_ARROW_UP || arrow_type == GTK_ARROW_DOWN)
+		height = sin (60.0 * 0.017453) * width;
+	else if (arrow_type == GTK_ARROW_LEFT || GTK_ARROW_RIGHT)
+		width = sin (60.0 * 0.017453) * height;
+
+
+	cr = ge_gdk_drawable_to_cairo (window, area);
+
+	switch (arrow_type)
 	{
-	  draw_shadow (style, window, GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-		       NULL, widget, "entry", x - 2, 0,
-		       width + 4, window_height);
+		case GTK_ARROW_UP:
+			cairo_move_to (cr, x, y + height);
+			cairo_line_to (cr, x + width, y + height);
+			cairo_line_to (cr, x + (width / 2.0), y);
+			cairo_line_to (cr, x, y + height);
+			break;
+		case GTK_ARROW_DOWN:
+			cairo_move_to (cr, x, y);
+			cairo_line_to (cr, x + width, y);
+			cairo_line_to (cr, x + (width / 2.0), y + height);
+			cairo_line_to (cr, x, y);
+			break;
+		case GTK_ARROW_LEFT:
+			cairo_move_to (cr, x + width, y);
+			cairo_line_to (cr, x, y + (height / 2.0));
+			cairo_line_to (cr, x + width, y + height);
+			cairo_line_to (cr, x + width, y);
+			break;
+		case GTK_ARROW_RIGHT:
+			cairo_move_to (cr, x, y);
+			cairo_line_to (cr, x + width, y + (height / 2.0));
+			cairo_line_to (cr, x, y + height);
+			cairo_line_to (cr, x, y);
+			break;
 	}
-    }
-    else
-    {
-	if (widget != 0 && !GTK_WIDGET_IS_SENSITIVE (widget))
-	    state_type = GTK_STATE_INSENSITIVE;
 
-	paint_arrow (window, style->fg_gc[state_type],
-		     arrow_type, x, y, width, height);
-    }
+	gdk_cairo_set_source_color (cr, &style->fg[state_type]);
+	cairo_fill (cr);
+	
+	cairo_destroy (cr);
+
 }
 
-#if 0
-static void
-draw_diamond (GtkStyle *style,
-	      GdkWindow *window,
-	      GtkStateType state_type,
-	      GtkShadowType shadow_type,
-	      GdkRectangle *area,
-	      GtkWidget *widget,
-	      const gchar *detail, gint x, gint y, gint width, gint height)
-{
-    gint half_width;
-    gint half_height;
-
-    CHECK_ARGS
-    SANITIZE_SIZE
-
-    debug ("draw_diamond: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
-	    detail, state_type, shadow_type, x, y, width, height);
-
-    half_width = width / 2;
-    half_height = height / 2;
-
-    if (area)
-    {
-	gdk_gc_set_clip_rectangle (style->light_gc[state_type], area);
-	gdk_gc_set_clip_rectangle (style->bg_gc[state_type], area);
-	gdk_gc_set_clip_rectangle (style->dark_gc[state_type], area);
-	gdk_gc_set_clip_rectangle (style->black_gc, area);
-    }
-    switch (shadow_type)
-    {
-    case GTK_SHADOW_IN:
-	gdk_draw_line (window, style->light_gc[state_type],
-		       x + 2, y + half_height,
-		       x + half_width, y + height - 2);
-	gdk_draw_line (window, style->light_gc[state_type],
-		       x + half_width, y + height - 2,
-		       x + width - 2, y + half_height);
-	gdk_draw_line (window, style->light_gc[state_type],
-		       x + 1, y + half_height,
-		       x + half_width, y + height - 1);
-	gdk_draw_line (window, style->light_gc[state_type],
-		       x + half_width, y + height - 1,
-		       x + width - 1, y + half_height);
-	gdk_draw_line (window, style->light_gc[state_type],
-		       x, y + half_height, x + half_width, y + height);
-	gdk_draw_line (window, style->light_gc[state_type],
-		       x + half_width, y + height,
-		       x + width, y + half_height);
-
-	gdk_draw_line (window, style->dark_gc[state_type],
-		       x + 2, y + half_height, x + half_width, y + 2);
-	gdk_draw_line (window, style->dark_gc[state_type],
-		       x + half_width, y + 2, x + width - 2, y + half_height);
-	gdk_draw_line (window, style->dark_gc[state_type], x + 1,
-		       y + half_height, x + half_width, y + 1);
-	gdk_draw_line (window, style->dark_gc[state_type], x + half_width,
-		       y + 1, x + width - 1, y + half_height);
-	gdk_draw_line (window, style->dark_gc[state_type], x,
-		       y + half_height, x + half_width, y);
-	gdk_draw_line (window, style->dark_gc[state_type], x + half_width,
-		       y, x + width, y + half_height);
-	break;
-    case GTK_SHADOW_OUT:
-	gdk_draw_line (window, style->dark_gc[state_type],
-		       x + 2, y + half_height,
-		       x + half_width, y + height - 2);
-	gdk_draw_line (window, style->dark_gc[state_type],
-		       x + half_width, y + height - 2,
-		       x + width - 2, y + half_height);
-	gdk_draw_line (window, style->dark_gc[state_type],
-		       x + 1, y + half_height,
-		       x + half_width, y + height - 1);
-	gdk_draw_line (window, style->dark_gc[state_type],
-		       x + half_width, y + height - 1,
-		       x + width - 1, y + half_height);
-	gdk_draw_line (window, style->dark_gc[state_type],
-		       x, y + half_height, x + half_width, y + height);
-	gdk_draw_line (window, style->dark_gc[state_type],
-		       x + half_width, y + height,
-		       x + width, y + half_height);
-
-	gdk_draw_line (window, style->light_gc[state_type],
-		       x + 2, y + half_height, x + half_width, y + 2);
-	gdk_draw_line (window, style->light_gc[state_type],
-		       x + half_width, y + 2, x + width - 2, y + half_height);
-	gdk_draw_line (window, style->light_gc[state_type], x + 1,
-		       y + half_height, x + half_width, y + 1);
-	gdk_draw_line (window, style->light_gc[state_type], x + half_width,
-		       y + 1, x + width - 1, y + half_height);
-	gdk_draw_line (window, style->light_gc[state_type], x,
-		       y + half_height, x + half_width, y);
-	gdk_draw_line (window, style->light_gc[state_type], x + half_width,
-		       y, x + width, y + half_height);
-	break;
-    default:
-	break;
-    }
-    if (area)
-    {
-	gdk_gc_set_clip_rectangle (style->light_gc[state_type], NULL);
-	gdk_gc_set_clip_rectangle (style->bg_gc[state_type], NULL);
-	gdk_gc_set_clip_rectangle (style->dark_gc[state_type], NULL);
-	gdk_gc_set_clip_rectangle (style->black_gc, NULL);
-    }
-}
-
-static void
-draw_string (GtkStyle *style,
-	     GdkWindow *window,
-	     GtkStateType state_type,
-	     GdkRectangle *area,
-	     GtkWidget *widget,
-	     const gchar *detail, gint x, gint y, const gchar *string)
-{
-    CHECK_ARGS
-
-    debug ("draw_string: detail=%s state=%d x=%d y=%d\n",
-	    detail, state_type, x, y);
-
-    if (area)
-    {
-	gdk_gc_set_clip_rectangle (style->white_gc, area);
-	gdk_gc_set_clip_rectangle (style->fg_gc[state_type], area);
-    }
-
-    /* FIXME GNOME2
-    if (state_type == GTK_STATE_INSENSITIVE)
-	gdk_draw_string (window, style->font, style->white_gc, x + 1, y + 1,
-			 string);
-    gdk_draw_string (window, style->font, style->fg_gc[state_type], x, y,
-		     string);
-    */
-    if (area)
-    {
-	gdk_gc_set_clip_rectangle (style->white_gc, NULL);
-	gdk_gc_set_clip_rectangle (style->fg_gc[state_type], NULL);
-    }
-}
-
-static void
-draw_flat_box (GtkStyle *style,
-	       GdkWindow *window,
-	       GtkStateType state_type,
-	       GtkShadowType shadow_type,
-	       GdkRectangle *area,
-	       GtkWidget *widget,
-	       const gchar *detail, gint x, gint y, gint width, gint height)
-{
-    GdkGC *gc1;
-
-    CHECK_ARGS
-    SANITIZE_SIZE
-
-    debug ("draw_flat_box: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
-	    detail, state_type, shadow_type, x, y, width, height);
-
-    gc1 = style->bg_gc[state_type];
-
-    if (DETAIL ("text") && (state_type == GTK_STATE_SELECTED))
-	gc1 = style->bg_gc[GTK_STATE_SELECTED];
-    else if (DETAIL ("viewportbin"))
-	gc1 = style->bg_gc[GTK_STATE_NORMAL];
-    else if (DETAIL ("entry_bg"))
-	gc1 = style->white_gc;
-
-    if ((!style->bg_pixmap[state_type])
-	|| (gc1 != style->bg_gc[state_type])
-	/* FIXME GNOME2 || (gdk_window_get_type (window) == GDK_WINDOW_PIXMAP)*/ )
-    {
-	if (area)
-	{
-	    gdk_gc_set_clip_rectangle (gc1, area);
-	}
-	gdk_draw_rectangle (window, gc1, TRUE, x, y, width, height);
-	if ((detail) && (!strcmp ("tooltip", detail)))
-	    gdk_draw_rectangle (window, style->black_gc, FALSE,
-				x, y, width - 1, height - 1);
-	if (area)
-	{
-	    gdk_gc_set_clip_rectangle (gc1, NULL);
-	}
-    }
-    else
-	gtk_style_apply_default_pixmap (style, window, state_type, area, x,
-					y, width, height);
-}
-#endif
-
 static void
 draw_check (GtkStyle *style,
 	    GdkWindow *window,
@@ -1607,9 +1169,8 @@ draw_check (GtkStyle *style,
 			cx+=1.0; cy+=1.0; cw -= 2.0; ch -= 2.0;
 		}
 	}
-	if (shadow_type != GTK_SHADOW_OUT)
+	if (shadow_type == GTK_SHADOW_IN)
 	{
-
 		cairo_set_line_width (cr, 2.0);
 
 		/* draw check mark shadow first */
@@ -1635,6 +1196,16 @@ draw_check (GtkStyle *style,
 		cairo_line_to (cr, cx + cw, cy + floor(ch / 4));
 		cairo_stroke (cr);
 	}
+	else if (shadow_type != GTK_SHADOW_OUT)
+	{
+		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.3);
+		cairo_rectangle (cr, x + width / 4, y + (height /3) + 0.5, width - (width / 2), height / 4);
+		cairo_fill (cr);
+
+		gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_SELECTED]);
+		cairo_rectangle (cr, x + width / 4, y + (height /3), width - (width / 2), height / 4);
+		cairo_fill (cr);
+	}
 
 	cairo_destroy (cr);
 }
@@ -1732,6 +1303,18 @@ draw_option (GtkStyle *style,
 			cairo_stroke (cr);
 		}
 	}
+	else if (shadow_type != GTK_SHADOW_OUT)
+	{
+		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.3);
+		cairo_rectangle (cr, x + width / 4, y + (height /3) + 0.5, width - (width / 2), height / 4);
+		cairo_fill (cr);
+
+		gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_SELECTED]);
+		cairo_rectangle (cr, x + width / 4, y + (height /3), width - (width / 2), height / 4);
+		cairo_fill (cr);
+	}
+
+
 
 	cairo_destroy (cr);
 }
@@ -1751,12 +1334,14 @@ draw_tab (GtkStyle *style,
 	/* Draws an option menu tab (the up and down pointing arrows)
 	 * TODO: Make this look neater
 	 */
+    	debug ("draw_tab: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
+	    detail, state_type, shadow_type, x, y, width, height);
 
 	draw_arrow (style, window, state_type, shadow_type, area, widget, detail,
-		    GTK_ARROW_UP, FALSE, x, y - 2, width, height);
+		    GTK_ARROW_UP, FALSE, x + width / 2, y - height / 2, height, height - 1);
 
 	draw_arrow (style, window, state_type, shadow_type, area, widget, detail,
-		    GTK_ARROW_DOWN, FALSE, x, y + 3, width, height);
+		    GTK_ARROW_DOWN, FALSE, x + width / 2, y + height / 2 + 1, height, height - 1);
 }
 
 static void
@@ -1848,6 +1433,18 @@ draw_box_gap (GtkStyle *style,
 	gap_width -= (1 - gap_x);
 	gap_x = 1;
     }
+    switch (gap_side)
+    {
+    case GTK_POS_TOP:
+    case GTK_POS_BOTTOM:
+      gap_width = MIN (gap_width, width - gap_x - 1);
+      break;
+    case GTK_POS_LEFT:
+    case GTK_POS_RIGHT:
+      gap_width = MIN (gap_width, height - gap_x - 1);
+      break;
+    }
+
 
     switch (gap_side)
     {
@@ -1892,25 +1489,15 @@ draw_extension (GtkStyle *style,
 		gint x,
 		gint y, gint width, gint height, GtkPositionType gap_side)
 {
-    GdkRectangle rect;
-
-    eazel_theme_data *theme_data;
-
-    CHECK_ARGS
-    SANITIZE_SIZE
-
-    theme_data = CRUX_RC_STYLE (style->rc_style)->theme_data;
-    g_assert (theme_data != NULL);
-
     debug ("draw_extension: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, x, y, width, height);
 
 	/* Get x relative to parent widget, not window */
 	GtkWidget* parent_widget;
 	gint relative_x;
-	if (widget != NULL)
+	if (widget != NULL &&
+	    (parent_widget = gtk_widget_get_parent(widget)) != NULL)
 	{
-		parent_widget = gtk_widget_get_parent(widget);
 		relative_x = x - parent_widget->allocation.x;
 		if (GTK_IS_CONTAINER(widget))
 			relative_x = relative_x - gtk_container_get_border_width((GtkContainer*)widget);
@@ -1920,32 +1507,6 @@ draw_extension (GtkStyle *style,
 		relative_x = x;
 	}
 
-#if 0
-    if (DETAIL ("tab"))
-    {
-	eazel_engine_stock_image type = 0;
-	switch (gap_side)
-	{
-	case GTK_POS_TOP:
-	    break;
-
-	case GTK_POS_BOTTOM:
-	    break;
-
-	default:			/* gcc drugging */
-	  ;
-	}
-	paint_background_area (style, window, state_type, area,
-			       x, y, width, height);
-	if (type != 0)
-	{
-	    paint_stock_image (theme_data, type, TRUE, FALSE, style, window,
-			       state_type, area, widget, x, y, width, height);
-	    return;
-	}
-    }
-#endif
-
 	cairo_t *cr;
 	cairo_pattern_t *crp;
 	CairoColor c1, c2;
@@ -2006,7 +1567,10 @@ draw_extension (GtkStyle *style,
 	
 	x++; y++; width -= 2.0; height -= 2.0;
 	ge_cairo_rounded_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0, 1.0, corners);
-	gdk_cairo_set_source_color (cr, &style->light[state_type]);
+	if (gap_side == GTK_POS_TOP)
+		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.2);
+	else
+		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.5); /* must be same colour as corresponding draw_box highlight */
 	cairo_stroke (cr);
 	cairo_destroy (cr);
 }
@@ -2019,14 +1583,6 @@ draw_focus (GtkStyle *style,
 	    GtkWidget *widget,
 	    const gchar *detail, gint x, gint y, gint width, gint height)
 {
-    eazel_theme_data *theme_data;
-
-    CHECK_ARGS
-    SANITIZE_SIZE
-
-    theme_data = CRUX_RC_STYLE (style->rc_style)->theme_data;
-    g_assert (theme_data != NULL);
-
     debug ("draw_focus: detail=%s x=%d y=%d w=%d h=%d\n",
 	    detail, x, y, width, height);
 
@@ -2056,70 +1612,57 @@ draw_slider (GtkStyle *style,
 	     gint x,
 	     gint y, gint width, gint height, GtkOrientation orientation)
 {
-    eazel_theme_data *theme_data;
-    gboolean focused;
+	cairo_t *cr;
+	cairo_pattern_t *crp;
+	CairoColor c1, c2;
 
-    CHECK_ARGS
-    SANITIZE_SIZE
+	CHECK_ARGS
+	SANITIZE_SIZE
 
-    theme_data = CRUX_RC_STYLE (style->rc_style)->theme_data;
-    g_assert (theme_data != NULL);
+	debug ("draw_slider: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n", detail, state_type, shadow_type, x, y, width, height);
 
-    debug ("draw_slider: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
-	    detail, state_type, shadow_type, x, y, width, height);
+	cr = ge_gdk_drawable_to_cairo (window, area);
 
-    if (area)
-	gdk_gc_set_clip_rectangle (style->black_gc, area);
+	cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0);
 
-    focused = (widget != 0);
+	if (orientation == GTK_ORIENTATION_VERTICAL)
+		crp = cairo_pattern_create_linear (x, y, x + width, y);
+	else
+		crp = cairo_pattern_create_linear (x, y, x, y + height);
 
-	{
-		cairo_t *cr;
-		cairo_pattern_t *crp;
-		CairoColor c1, c2;
+	ge_gdk_color_to_cairo (&style->bg[GTK_STATE_SELECTED], &c2);
 
-		cr = ge_gdk_drawable_to_cairo (window, area);
+	/* light */
+	CRUX_LIGHT (c2, c1)
 
-		cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0);
+	/* dark */
+	CRUX_DARK (c2, c2)
 
-		if (orientation == GTK_ORIENTATION_VERTICAL)
-			crp = cairo_pattern_create_linear (x, y, x + width, y);
-		else
-			crp = cairo_pattern_create_linear (x, y, x, y + height);
+	cairo_pattern_add_color_stop_rgb (crp, 0.0, c1.r, c1.g, c1.b);
+	cairo_pattern_add_color_stop_rgb (crp, 1.0, c2.r, c2.g, c2.b);
+	cairo_set_source (cr, crp);
+	cairo_fill_preserve (cr);
+	cairo_pattern_destroy (crp);
 
-		ge_gdk_color_to_cairo (&style->light[GTK_STATE_SELECTED], &c1);
-		ge_gdk_color_to_cairo (&style->dark[GTK_STATE_SELECTED], &c2);
-		cairo_pattern_add_color_stop_rgb (crp, 0.0, c1.r, c1.g, c1.b);
-		cairo_pattern_add_color_stop_rgb (crp, 1.0, c2.r, c2.g, c2.b);
-		cairo_set_source (cr, crp);
-		cairo_fill_preserve (cr);
-		cairo_pattern_destroy (crp);
-
-		gdk_cairo_set_source_color (cr, &style->dark[GTK_STATE_SELECTED]);
-		cairo_stroke (cr);
-/*
-		cairo_rectangle (cr, x + 1.5, y + 1.5, width - 2.0, height - 2.0);
-		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.3);
-		cairo_stroke (cr);
-*/
-		/* bevel */
-		cairo_rectangle (cr, x+1.5, y+1.5, width-3.0, height-3.0);
-		if (orientation == GTK_ORIENTATION_VERTICAL)
-			crp = cairo_pattern_create_linear (x, y, x + width, y);
-		else
-			crp = cairo_pattern_create_linear (x, y, x, y + height);
-		cairo_pattern_add_color_stop_rgba (crp, 0.0, 1.0, 1.0, 1.0, .5);
-		cairo_pattern_add_color_stop_rgba (crp, 1.0, 1.0, 1.0, 1.0, 0.2);
-		cairo_set_source (cr, crp);
-		cairo_stroke (cr);
-
-		cairo_pattern_destroy (crp);
+	cairo_set_source_rgb (cr, c2.r, c2.g, c2.b);
+	cairo_stroke (cr);
 
 
-		cairo_destroy (cr);
-	}
-    if (area)
-	gdk_gc_set_clip_rectangle (style->black_gc, NULL);
+	/* bevel */
+	cairo_rectangle (cr, x+1.5, y+1.5, width-3.0, height-3.0);
+	if (orientation == GTK_ORIENTATION_VERTICAL)
+		crp = cairo_pattern_create_linear (x, y, x + width, y);
+	else
+		crp = cairo_pattern_create_linear (x, y, x, y + height);
+	cairo_pattern_add_color_stop_rgba (crp, 0.0, 1.0, 1.0, 1.0, 0.2);
+	cairo_pattern_add_color_stop_rgba (crp, 1.0, 1.0, 1.0, 1.0, 0.1);
+	cairo_set_source (cr, crp);
+	cairo_stroke (cr);
+
+	cairo_pattern_destroy (crp);
+
+
+	cairo_destroy (cr);
 }
 
 static void
@@ -2155,8 +1698,8 @@ draw_handle (GtkStyle *style,
 	cairo_destroy (cr);
 	*/
 
-
-	draw_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
+	if (!ge_is_panel_widget_item (widget) && !ge_object_is_a (widget, "PanelToplevel"))
+		draw_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
 
     light_gc = style->light_gc[state_type];
     dark_gc = style->dark_gc[state_type];
@@ -2200,6 +1743,37 @@ draw_handle (GtkStyle *style,
     gdk_gc_set_clip_rectangle (dark_gc, NULL);
 }
 
+static void
+draw_layout (GtkStyle        *style,
+	     GdkWindow       *window,
+	     GtkStateType     state_type,
+	     gboolean         use_text,
+	     GdkRectangle    *area,
+	     GtkWidget       *widget,
+	     const char      *detail,
+	     int              x,
+	     int              y,
+	     PangoLayout      *layout)
+{
+	GdkGC *gc;
+	
+	g_return_if_fail (GTK_IS_STYLE (style));
+	g_return_if_fail (window != NULL);
+	
+	gc = use_text ? style->text_gc[state_type] : style->fg_gc[state_type];
+	
+	if (area) {
+		gdk_gc_set_clip_rectangle (gc, area);
+	}
+	
+	gdk_draw_layout (window, gc, x, y, layout);
+	
+	if (area) {
+		gdk_gc_set_clip_rectangle (gc, NULL);
+	}
+}
+
+
 void
 crux_draw_style_class_init (GtkStyleClass *style_class)
 {
@@ -2209,12 +1783,8 @@ crux_draw_style_class_init (GtkStyleClass *style_class)
   style_class->draw_hline = draw_hline;
   style_class->draw_vline = draw_vline;
   style_class->draw_shadow = draw_shadow;
-  //style_class->draw_polygon = draw_polygon;
   style_class->draw_arrow = draw_arrow;
-  //style_class->draw_diamond = draw_diamond;
-  //style_class->draw_string = draw_string;
   style_class->draw_box = draw_box;
-  //style_class->draw_flat_box = draw_flat_box;
   style_class->draw_check = draw_check;
   style_class->draw_option = draw_option;
   style_class->draw_tab = draw_tab;
@@ -2224,4 +1794,5 @@ crux_draw_style_class_init (GtkStyleClass *style_class)
   style_class->draw_focus = draw_focus;
   style_class->draw_slider = draw_slider;
   style_class->draw_handle = draw_handle;
+  style_class->draw_layout = draw_layout;
 }
