@@ -19,6 +19,7 @@
  * Written by Andrew Johnson <acjgenius@earthlink.net>
  */ 
 
+#include <string.h>
 #include "glide_gtk2_engine.h"
 #include "glide_gtk2_support.h"
 #include "glide_gtk2_drawing.h"
@@ -40,9 +41,11 @@ glide_draw_hline (GtkStyle * style,
             gint y)
 {
   GlideStyle *glide_style = GLIDE_STYLE (style);
+  cairo_t *canvas;
+
   CHECK_ARGS
  
-  cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+  canvas = ge_gdk_drawable_to_cairo (window, area);
 
   do_glide_draw_line(canvas, 
                        &glide_style->color_cube.dark[state_type], 
@@ -71,12 +74,14 @@ glide_draw_vline (GtkStyle * style,
             gint x)
 {
   GlideStyle *glide_style = GLIDE_STYLE (style);
+  cairo_t *canvas;
+
   if (ge_is_combo_box(widget, FALSE) && (!ge_is_combo_box_entry(widget)))
     return;
  
   CHECK_ARGS
  
-  cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+  canvas = ge_gdk_drawable_to_cairo (window, area);
 
   do_glide_draw_line(canvas, 
                        &glide_style->color_cube.dark[state_type], 
@@ -107,6 +112,8 @@ glide_draw_check (GtkStyle * style,
             gint height)
 {
 	GlideStyle *glide_style = GLIDE_STYLE (style);
+	cairo_t *canvas;
+	gboolean inconsistent;
 
 	CHECK_ARGS
 	SANITIZE_SIZE
@@ -118,7 +125,7 @@ glide_draw_check (GtkStyle * style,
 		width += 2;
 		height += 2;
 	} 
-	else if (GTK_IS_CELL_RENDERER_TOGGLE(widget) || CHECK_DETAIL("cellcheck",detail))
+	else if (GTK_IS_CELL_RENDERER_TOGGLE(widget) || CHECK_DETAIL(detail, "cellcheck"))
 	{
 		x -= 1;
 		y -= 1;
@@ -127,7 +134,7 @@ glide_draw_check (GtkStyle * style,
 	}
 
  
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	ge_cairo_set_color(canvas, &glide_style->color_cube.base[state]);
 
@@ -135,10 +142,10 @@ glide_draw_check (GtkStyle * style,
 
 	cairo_fill(canvas);
 
-	gboolean inconsistent = (IS_TOGGLE_BUTTON(widget) && gtk_toggle_button_get_inconsistent(TOGGLE_BUTTON(widget)));
+	inconsistent = (IS_TOGGLE_BUTTON(widget) && gtk_toggle_button_get_inconsistent(TOGGLE_BUTTON(widget)));
 
-	inconsistent |= (GTK_IS_CELL_RENDERER_TOGGLE(widget) && gtk_cell_renderer_toggle_get_inconsistent (widget));
-	inconsistent |= (CHECK_DETAIL("cellcheck",detail) && (shadow == GTK_SHADOW_ETCHED_IN));
+	inconsistent |= (GTK_IS_CELL_RENDERER_TOGGLE(widget) && ge_cell_renderer_toggle_get_inconsistent (widget));
+	inconsistent |= (CHECK_DETAIL(detail, "cellcheck") && (shadow == GTK_SHADOW_ETCHED_IN));
 
 	if ((shadow == GTK_SHADOW_IN) || (inconsistent))
 	{
@@ -208,6 +215,21 @@ glide_draw_option (GtkStyle * style,
              gint width, 
              gint height)
 {
+	GlideCheckState check_state;
+	gboolean observe_shadow;
+	gboolean draw_check;
+
+	GlideStyle *glide_style;
+ 
+	CairoColor *base;
+	CairoColor *fill_color;
+	CairoColor *check_color; 
+
+	GlideBevelStyle bevel_style;
+	GlideBorderType border_type;
+
+	cairo_t *canvas;
+
 	/***********************************************/
 	/* GTK Sanity Checks                           */
 	/***********************************************/
@@ -218,7 +240,7 @@ glide_draw_option (GtkStyle * style,
 	/***********************************************/
 	/* GTK Special Cases - adjust Size/Offset      */
 	/***********************************************/
-	if (GTK_IS_CELL_RENDERER_TOGGLE(widget) || CHECK_DETAIL("cellradio",detail))
+	if (GTK_IS_CELL_RENDERER_TOGGLE(widget) || CHECK_DETAIL(detail, "cellradio"))
         {
 		x -= 1;
 		y -= 1;
@@ -230,19 +252,19 @@ glide_draw_option (GtkStyle * style,
 	/***********************************************/
 	/* GTK Special Cases - Determine Check State   */
 	/***********************************************/
-	GlideCheckState check_state = GLIDE_CHECK_OFF;
+	check_state = GLIDE_CHECK_OFF;
 
-	if (IS_TOGGLE_BUTTON(widget) && gtk_toggle_button_get_inconsistent(TOGGLE_BUTTON(widget)))
+	if (GTK_IS_TOGGLE_BUTTON(widget) && gtk_toggle_button_get_inconsistent(TOGGLE_BUTTON(widget)))
 	{
 		check_state = GLIDE_CHECK_INCONSISTENT;
 	}
 
-	if (GTK_IS_CELL_RENDERER_TOGGLE(widget) && gtk_cell_renderer_toggle_get_inconsistent (widget))
+	if (GTK_IS_CELL_RENDERER_TOGGLE(widget) && ge_cell_renderer_toggle_get_inconsistent (widget))
 	{
 		check_state = GLIDE_CHECK_INCONSISTENT;
 	}
 
-	if (CHECK_DETAIL("cellradio",detail) && (shadow == GTK_SHADOW_ETCHED_IN))
+	if (CHECK_DETAIL(detail, "cellradio") && (shadow == GTK_SHADOW_ETCHED_IN))
 	{
 		check_state = GLIDE_CHECK_INCONSISTENT;
 	}
@@ -256,18 +278,18 @@ glide_draw_option (GtkStyle * style,
 	/***********************************************/
 	/* Appearance                                  */
 	/***********************************************/
-	gboolean observe_shadow = FALSE;
-	gboolean draw_check = !observe_shadow;
+	observe_shadow = FALSE;
+	draw_check = !observe_shadow;
 
 
 		/***********************************************/
 		/* Colors                                      */
 		/***********************************************/
-		GlideStyle *glide_style = GLIDE_STYLE (style);
+		glide_style = GLIDE_STYLE (style);
  
-		CairoColor *base = &glide_style->color_cube.bg[state];
-		CairoColor *fill_color = &glide_style->color_cube.base[state];
-		CairoColor *check_color = &glide_style->color_cube.text[state]; 
+		base = &glide_style->color_cube.bg[state];
+		fill_color = &glide_style->color_cube.base[state];
+		check_color = &glide_style->color_cube.text[state]; 
 
 		if (state == GTK_STATE_INSENSITIVE)
 		{
@@ -283,8 +305,8 @@ glide_draw_option (GtkStyle * style,
 		/***********************************************/
 		/* Style                                       */
 		/***********************************************/
-		GlideBevelStyle bevel_style = GLIDE_BEVEL_STYLE_SMOOTHER;
-		GlideBorderType border_type = GLIDE_BORDER_TYPE_IN;
+		bevel_style = GLIDE_BEVEL_STYLE_SMOOTHER;
+		border_type = GLIDE_BORDER_TYPE_IN;
 
 		if (observe_shadow)
 		{
@@ -326,7 +348,7 @@ glide_draw_option (GtkStyle * style,
 	/***********************************************/
 	/* Draw Option                                 */
 	/***********************************************/
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	do_glide_draw_round_option(canvas, base, fill_color, check_color, 
 					bevel_style, border_type, check_state,
@@ -358,6 +380,13 @@ glide_draw_arrow (GtkStyle * style,
             gint width, 
             gint height)
 {
+	gboolean button_in;
+	gint child_offset_x, child_offset_y;
+
+	GlideStyle *glide_style;
+
+	cairo_t *canvas;
+
 	/***********************************************/
 	/* GTK Sanity Checks                           */
 	/***********************************************/
@@ -367,8 +396,8 @@ glide_draw_arrow (GtkStyle * style,
 	/***********************************************/
 	/* GTK Arrow Special Cases - adjust Size/Offset*/
 	/***********************************************/
-	gboolean button_in = (shadow == GTK_SHADOW_IN);
-	gint child_offset_x = 1, child_offset_y = 1;
+	button_in = (shadow == GTK_SHADOW_IN);
+	child_offset_x = child_offset_y = 1;
  
 	if ((ge_is_combo_box(widget, FALSE) && !ge_is_combo_box_entry(widget) ) ||
 		(CHECK_DETAIL (detail, "spinbutton")))
@@ -453,9 +482,9 @@ glide_draw_arrow (GtkStyle * style,
 	/***********************************************/
 	/* Draw Arrow                                  */
 	/***********************************************/
-	GlideStyle *glide_style = GLIDE_STYLE (style);
+	glide_style = GLIDE_STYLE (style);
 
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	if (state == GTK_STATE_INSENSITIVE)
 	{
@@ -493,13 +522,19 @@ glide_draw_shadow (GtkStyle * style,
              gint width, 
              gint height)
 {
+	GlideBevelStyle bevel_style;
+	GlideBorderType border_type;
+
+	cairo_t *canvas;
+ 	CairoColor *background;
+
 	CHECK_ARGS
 	SANITIZE_SIZE
 
 	/***********************************************/
 	/* GTK Special Cases - adjust Size/Type        */
 	/***********************************************/
-	GlideBevelStyle bevel_style = GLIDE_BEVEL_STYLE_DEFAULT;
+	bevel_style = GLIDE_BEVEL_STYLE_DEFAULT;
 	if (CHECK_DETAIL (detail, "buttondefault"))
 	{
 		return;
@@ -550,8 +585,6 @@ glide_draw_shadow (GtkStyle * style,
 	/***********************************************/
 	/* Convert Shadow Type To Border Type          */
 	/***********************************************/
-	GlideBorderType border_type;
-
 	switch (shadow_type)
 	{
 		case GTK_SHADOW_ETCHED_IN:
@@ -578,8 +611,8 @@ glide_draw_shadow (GtkStyle * style,
 	/***********************************************/
 	/* Draw Border                                 */
 	/***********************************************/
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
- 	CairoColor *background = &GLIDE_STYLE (style)->color_cube.bg[state_type];
+	canvas = ge_gdk_drawable_to_cairo (window, area);
+ 	background = &GLIDE_STYLE (style)->color_cube.bg[state_type];
 
 	do_glide_draw_border(canvas, background, bevel_style, border_type, x, y, width, height);
 
@@ -670,6 +703,8 @@ glide_draw_combobox_button (GtkStyle * style,
 
   if ((!(widget)) || (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR))
     {
+      cairo_t *canvas;
+
       if (ge_is_combo_box_entry (widget)
           || ge_is_combo_box (widget, TRUE))
         {
@@ -693,8 +728,7 @@ glide_draw_combobox_button (GtkStyle * style,
 
             }
           else
-#warning FIXME - gtk_paint_flat_box
-            gtk_paint_flat_box (style, window, parent_state,
+            glide_draw_flat_box(style, window, parent_state,
 			        GTK_SHADOW_NONE, area, widget, "entry_bg", x - 2,
 			        y, width + 2, height);
  
@@ -728,8 +762,7 @@ glide_draw_combobox_button (GtkStyle * style,
  
             parent_state = GTK_STATE_NORMAL;
 
-#warning FIXME - gtk_paint_flat_box
-          gtk_paint_flat_box (parent_style, window, parent_state,
+          glide_draw_flat_box (parent_style, window, parent_state,
 	 		      GTK_SHADOW_NONE, area, entry, "entry_bg", x - 2,
 			      y, width + 2, height);
 
@@ -770,8 +803,7 @@ glide_draw_combobox_button (GtkStyle * style,
           if (parent_state != GTK_STATE_INSENSITIVE)
             parent_state = GTK_STATE_NORMAL;
 
-#warning FIXME - gtk_paint_flat_box
-          gtk_paint_flat_box (parent_style, window, parent_state,
+          glide_draw_flat_box (parent_style, window, parent_state,
 			      GTK_SHADOW_NONE, area, parent, "entry_bg", x - 2,
 			      y, width + 2, height);
           glide_draw_shadow (parent_style, window, parent_state, GTK_SHADOW_IN,
@@ -779,7 +811,7 @@ glide_draw_combobox_button (GtkStyle * style,
 				width + thick*2 + focus_padding*2, height - focus*2 + focus_padding*2);
         }
  
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	glide_draw_pattern_fill(canvas, DEFAULT_BACKGROUND_PATTERN(glide_style, state_type, &glide_style->bg_gradient[FALSE][state_type]), x + focus, y + thick + focus,
                              width - thick - focus*2, height - thick*2 - focus*2);
@@ -800,6 +832,8 @@ glide_draw_combobox_button (GtkStyle * style,
     } 
   else
     {
+      cairo_t *canvas;
+
       focus_padding = -focus_padding;
 
       if (ge_is_combo_box_entry (widget)
@@ -828,8 +862,7 @@ glide_draw_combobox_button (GtkStyle * style,
               if (parent_state != GTK_STATE_INSENSITIVE)
                 parent_state = GTK_STATE_NORMAL;
 
-#warning FIXME - gtk_paint_flat_box
-            gtk_paint_flat_box (style, window, parent_state,
+            glide_draw_flat_box (style, window, parent_state,
 	  	                GTK_SHADOW_NONE, area, widget, "entry_bg", 
 				x + focus - focus_padding, y + focus - focus_padding, 
 				width + thick*2 - focus + focus_padding*2, height - focus*2 + focus_padding*2);
@@ -867,8 +900,7 @@ glide_draw_combobox_button (GtkStyle * style,
            if (parent_state != GTK_STATE_INSENSITIVE)
              parent_state = GTK_STATE_NORMAL;
 
-#warning FIXME - gtk_paint_flat_box
-           gtk_paint_flat_box (parent_style, window, parent_state,
+           glide_draw_flat_box (parent_style, window, parent_state,
 			       GTK_SHADOW_NONE, area, entry, "entry_bg", 
 				x + focus - focus_padding, y + focus - focus_padding, 
 				width + thick*2 - focus + focus_padding*2, height - focus*2 + focus_padding*2);
@@ -895,8 +927,7 @@ glide_draw_combobox_button (GtkStyle * style,
           if (parent_state != GTK_STATE_INSENSITIVE)
             parent_state = GTK_STATE_NORMAL;
 
-#warning FIXME - gtk_paint_flat_box
-          gtk_paint_flat_box (parent_style, window, parent_state,
+          glide_draw_flat_box (parent_style, window, parent_state,
 			      GTK_SHADOW_NONE, area, parent, "entry_bg", 
 				x + focus - focus_padding, y + focus - focus_padding, 
 				width + thick*2 - focus + focus_padding*2, height - focus*2 + focus_padding*2);
@@ -906,7 +937,7 @@ glide_draw_combobox_button (GtkStyle * style,
                                 width + thick*2 - focus + focus_padding*2, height - focus*2 + focus_padding*2);
         }
  
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	glide_draw_pattern_fill(canvas, DEFAULT_BACKGROUND_PATTERN(glide_style, state_type, &glide_style->bg_gradient[FALSE][state_type]), x + thick + focus, y + thick + focus, width - thick - focus*2, height - 2*thick - focus*2);
 
@@ -959,17 +990,18 @@ glide_draw_spinbutton_stepper (GtkStyle * style,
    * more complicated because we can only do half for each stepper.
    */
  
+  GtkStateType parent_state = widget ? GTK_WIDGET_STATE(widget) : GTK_STATE_NORMAL;
+  gboolean entry_focused = FALSE;
+  gint focus_line_width = 0, focus_padding = 0, focus = 0, thick = 2;
+  gboolean interior_focus = TRUE;
+  cairo_t *canvas;
+
   spin_area.x = x;
   spin_area.y = y;
   spin_area.width = width;
   spin_area.height = height;
  
-  GtkStateType parent_state = GTK_WIDGET_STATE(widget);
-  gboolean entry_focused = FALSE;
-  gint focus_line_width = 0, focus_padding = 0, focus = 0;
-  gboolean interior_focus = TRUE;
-
-  if (GTK_WIDGET_HAS_FOCUS(widget))
+  if (widget && GTK_WIDGET_HAS_FOCUS(widget))
   {
 	gtk_widget_style_get (widget, "focus_line_width", &focus_line_width, NULL);
 	gtk_widget_style_get (widget, "focus_padding", &focus_padding, NULL);
@@ -991,14 +1023,11 @@ glide_draw_spinbutton_stepper (GtkStyle * style,
 	parent_state = GTK_STATE_NORMAL;
   }
 
-  gint thick = 2;
-
   if ((!(widget)) || (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR))
    {
       if (CHECK_DETAIL (detail, "spinbutton_up"))
 	{
-#warning FIXME - gtk_paint_flat_box
-	  gtk_paint_flat_box (style, window, parent_state,
+	  glide_draw_flat_box (style, window, parent_state,
 			      GTK_SHADOW_NONE, &spin_area, widget,
 			      "entry_bg", 
 				x - thick, y + focus, width + thick - focus, height - focus*2);
@@ -1006,7 +1035,7 @@ glide_draw_spinbutton_stepper (GtkStyle * style,
 	  glide_draw_shadow (style, window, parent_state, GTK_SHADOW_IN, &spin_area, widget, detail, 
                                 x - thick, y + focus, width + thick - focus, height + thick - focus);
 
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	if (!((width + thick) % 2) && !focus)
 	{
@@ -1020,7 +1049,7 @@ glide_draw_spinbutton_stepper (GtkStyle * style,
 	  glide_draw_shadow (style, window, state_type, shadow_type, area, widget, detail,
                                 x, y + thick + focus, width - thick - focus, height - thick - focus);
 
-	if ((widget->requisition.height / 2) != (widget->requisition.height + 1) / 2)
+	if (!widget || (widget->requisition.height / 2) != (widget->requisition.height + 1) / 2)
 	{
 		glide_draw_arrow (style, window, state_type, shadow_type, area, NULL, "spinbutton_arrow", GTK_ARROW_UP, TRUE,
 					x + thick/2, y + thick + 1, width - thick*2, height - thick*2 + 1);
@@ -1041,15 +1070,14 @@ glide_draw_spinbutton_stepper (GtkStyle * style,
 	}
       else
 	{
-#warning FIXME - gtk_paint_flat_box
-	  gtk_paint_flat_box (style, window, parent_state,
+	  glide_draw_flat_box (style, window, parent_state,
 			      GTK_SHADOW_NONE, &spin_area, widget,
 			      "entry_bg", x - thick, y, width + thick - focus, height - focus);
 
 	  glide_draw_shadow (style, window, parent_state, GTK_SHADOW_IN, &spin_area, widget, detail,  
 			    x - thick, y - thick, width + thick - focus, height + thick - focus);
 
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	if (!((width + thick) % 2) && !focus)
 	{
@@ -1062,7 +1090,7 @@ glide_draw_spinbutton_stepper (GtkStyle * style,
 	  glide_draw_shadow (style, window, state_type, shadow_type, area, widget, detail,
 			    x, y , width - thick - focus, height - thick - focus);
 
-	if ((widget->requisition.height / 2) != (widget->requisition.height + 1) / 2)
+	if (!widget || (widget->requisition.height / 2) != (widget->requisition.height + 1) / 2)
 	{
 		glide_draw_arrow (style, window, state_type, shadow_type, area, NULL, "spinbutton_arrow", GTK_ARROW_DOWN, TRUE,
 					x + thick/2, y + thick/2 + 1 - focus, width - thick*2, height - thick*2 - 2);
@@ -1086,14 +1114,13 @@ glide_draw_spinbutton_stepper (GtkStyle * style,
     {
       if (CHECK_DETAIL (detail, "spinbutton_up"))
 	{
-#warning FIXME - gtk_paint_flat_box
-	  gtk_paint_flat_box (style, window, parent_state,
+	  glide_draw_flat_box (style, window, parent_state,
 			      GTK_SHADOW_NONE, &spin_area, widget,
 			      "entry_bg", x + focus, y + focus, width + thick*2 - focus, height - focus);
 	  glide_draw_shadow (style, window, parent_state, GTK_SHADOW_IN, &spin_area, widget, detail, 
 				x + focus, y + focus, width + thick*2 - focus, height + thick - focus);
 
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	if (!((width + thick) % 2) && !focus)
 	{
@@ -1126,13 +1153,12 @@ glide_draw_spinbutton_stepper (GtkStyle * style,
 	}
       else
 	{
-#warning FIXME - gtk_paint_flat_box
-	  gtk_paint_flat_box (style, window, parent_state,
+	  glide_draw_flat_box (style, window, parent_state,
 			      GTK_SHADOW_NONE, &spin_area, widget,
 			      "entry_bg",x + focus, y, width + thick*2 - focus, height - focus);
 	  glide_draw_shadow (style, window, parent_state, GTK_SHADOW_IN, &spin_area, widget, detail, 
 				x + focus , y - thick, width + thick*2 - focus, height + thick - focus);
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	if (!((width + thick) % 2) && !focus)
 	{
@@ -1196,6 +1222,8 @@ glide_draw_box (GtkStyle * style,
           gint height)
 {
   GlideStyle *glide_style = GLIDE_STYLE (style);
+  GlideBorderType border_type;
+  cairo_t *canvas;
  
   CHECK_ARGS
   SANITIZE_SIZE
@@ -1212,8 +1240,6 @@ glide_draw_box (GtkStyle * style,
 	/***********************************************/
 	/* Convert Shadow Type To Border Type          */
 	/***********************************************/
-	GlideBorderType border_type;
-
 	switch (shadow_type)
 	{
 		case GTK_SHADOW_ETCHED_IN:
@@ -1237,7 +1263,7 @@ glide_draw_box (GtkStyle * style,
 		break;
 	}
 
-  cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+  canvas = ge_gdk_drawable_to_cairo (window, area);
 
   if ((CHECK_DETAIL (detail, "vscrollbar"))  || (CHECK_DETAIL (detail, "hscrollbar")))
     {
@@ -1245,7 +1271,7 @@ glide_draw_box (GtkStyle * style,
 
       glide_draw_pattern_fill(canvas, DEFAULT_BACKGROUND_PATTERN(glide_style, state_type, &glide_style->bg_gradient[vertical][state_type]), x-1, y-1, width+1, height+1);
  
-	do_glide_draw_border(canvas, &glide_style->color_cube.bg[GTK_STATE_NORMAL],
+	do_glide_draw_border(canvas, &glide_style->color_cube.bg[state_type],
 				GLIDE_BEVEL_STYLE_SMOOTHER, border_type,
 				x, y, width, height);
 
@@ -1256,11 +1282,12 @@ glide_draw_box (GtkStyle * style,
     {
 	if (GTK_IS_SCALE (widget)) 
 	{
-		glide_draw_pattern_fill(canvas, DEFAULT_BACKGROUND_PATTERN(glide_style, GTK_STATE_NORMAL, &glide_style->bg_solid[GTK_STATE_NORMAL]), x, y, width, height);
-
-		GtkAdjustment * adjustment = gtk_range_get_adjustment(GTK_RANGE(widget));
+		GtkAdjustment *adjustment;
 		gfloat value = 0;
   
+		glide_draw_pattern_fill(canvas, DEFAULT_BACKGROUND_PATTERN(glide_style, GTK_STATE_NORMAL, &glide_style->bg_solid[GTK_STATE_NORMAL]), x, y, width, height);
+
+		adjustment = gtk_range_get_adjustment(GTK_RANGE(widget));
 		value = gtk_range_get_value(GTK_RANGE(widget));
 
 		if GTK_IS_HSCALE (widget)
@@ -1566,6 +1593,46 @@ gboolean vertical = TRUE;
    cairo_destroy(canvas);
 }
  
+/* Draw Function For Boxes Traditionally Either Without Borders, 
+		or With A Single Pixel Line */
+
+void
+glide_draw_flat_box (GtkStyle * style,
+	  GdkWindow * window,
+	  GtkStateType state_type,
+	  GtkShadowType shadow_type,
+	  GdkRectangle * area,
+	  GtkWidget * widget,
+	  const gchar * detail, 
+          gint x, 
+          gint y, 
+          gint width, 
+          gint height)
+{
+	#warning FIXME - FINISH draw_flat_box
+	if (CHECK_DETAIL(detail, "tooltip"))
+	{
+		GlideStyle *glide_style = GLIDE_STYLE (style);
+
+		CHECK_ARGS
+		SANITIZE_SIZE
+
+		cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+
+		glide_draw_pattern_fill(canvas, DEFAULT_BACKGROUND_PATTERN(glide_style, state_type, &glide_style->bg_solid[state_type]), 
+						x, y, width, height);
+
+		do_glide_draw_border(canvas, &glide_style->color_cube.bg[state_type],
+					GLIDE_BEVEL_STYLE_FLAT, GLIDE_BORDER_TYPE_IN,
+					x, y, width - 1, height - 1);
+
+		cairo_destroy(canvas);
+	}
+	else
+		glide_parent_class->draw_flat_box (style, window, state_type, shadow_type, area,
+							widget, detail, x, y, width, height);
+}
+
 /***********************************************
  * glide_draw_tab -
  *  
@@ -1607,23 +1674,24 @@ glide_draw_slider (GtkStyle * style,
              GtkOrientation orientation)
 {
 	GlideStyle *glide_style = GLIDE_STYLE (style);
+	cairo_t *canvas;
  
 	CHECK_ARGS
 	SANITIZE_SIZE
  
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	glide_draw_pattern_fill(canvas,DEFAULT_BACKGROUND_PATTERN(glide_style, state_type, &glide_style->bg_gradient[orientation == GTK_ORIENTATION_VERTICAL][state_type]), x, y, width, height);
 
-	do_glide_draw_border(canvas, &glide_style->color_cube.bg[GTK_STATE_NORMAL],
+	do_glide_draw_border(canvas, &glide_style->color_cube.bg[state_type],
 				(IS_SCROLLBAR(widget))?GLIDE_BEVEL_STYLE_SMOOTHER:GLIDE_BEVEL_STYLE_DEFAULT, GLIDE_BORDER_TYPE_OUT,
 				x, y, width, height);
 
 	if (!IS_SCALE(widget))
 	{
 		do_glide_draw_grip (canvas,
-					&glide_style->color_cube.light[GTK_STATE_NORMAL],
-					&glide_style->color_cube.dark[GTK_STATE_NORMAL],
+					&glide_style->color_cube.light[state_type],
+					&glide_style->color_cube.dark[state_type],
 					x, y, width, height, orientation == GTK_ORIENTATION_VERTICAL);
 
 		glide_draw_pattern_fill(canvas, DEFAULT_OVERLAY_PATTERN(glide_style, detail,orientation == GTK_ORIENTATION_VERTICAL), x, y, width, height);
@@ -1653,11 +1721,12 @@ glide_draw_box_gap (GtkStyle * style,
 		 gint gap_size)
 {
 	GlideStyle *glide_style = GLIDE_STYLE (style);
+	cairo_t *canvas;
 
 	CHECK_ARGS
 	SANITIZE_SIZE
 
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	glide_draw_pattern_fill(canvas, DEFAULT_BACKGROUND_PATTERN(glide_style, state_type, &glide_style->bg_solid[state_type]), x, y, width, height);
 
@@ -1683,6 +1752,12 @@ glide_draw_shadow_gap (GtkStyle * style,
 		 gint gap_pos, 
 		 gint gap_size)
 {
+	GlideSide side;
+	GlideBorderType border_type;
+	GlideBevelStyle bevel_style;
+	cairo_t *canvas;
+
+
 	/***********************************************/
 	/* GTK Sanity Checks                           */
 	/***********************************************/
@@ -1695,7 +1770,7 @@ glide_draw_shadow_gap (GtkStyle * style,
 	/***********************************************/
 	/* GTK Special Cases - adjust Size/Offset      */
 	/***********************************************/
-	GlideSide side = GLIDE_SIDE_NONE;
+	side = GLIDE_SIDE_NONE;
 
 	if (gap_size > 0)
 	{
@@ -1765,8 +1840,6 @@ glide_draw_shadow_gap (GtkStyle * style,
 	/***********************************************/
 	/* Convert Shadow Type To Border Type          */
 	/***********************************************/
-	GlideBorderType border_type;
-
 	switch (shadow_type)
 	{
 		case GTK_SHADOW_ETCHED_IN:
@@ -1794,10 +1867,10 @@ glide_draw_shadow_gap (GtkStyle * style,
 	/***********************************************/
 	/* Draw Border                                 */
 	/***********************************************/
-	GlideBevelStyle bevel_style = GLIDE_BEVEL_STYLE_DEFAULT;
+	bevel_style = GLIDE_BEVEL_STYLE_DEFAULT;
 //	inner_overlap = (((gap_side==GTK_POS_RIGHT) || (gap_side==GTK_POS_BOTTOM)) && (gap_pos==0));
 
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	do_glide_draw_border_with_gap(canvas, 
 					&GLIDE_STYLE(style)->color_cube.bg[state_type], 
@@ -1836,12 +1909,19 @@ glide_draw_extension (GtkStyle * style,
 
 	gint widget_x = 0, widget_y = 0, widget_width = 0, widget_height = 0;
 
-	CHECK_ARGS
-	SANITIZE_SIZE
-
 	GdkRectangle clip, shadow;
 	CairoPattern pattern;
 
+	gint gap_pos = 0, gap_shrink = 0;
+
+	GlideSide side;
+	GlideBorderType border_type;
+	GlideBevelStyle bevel_style;
+	cairo_t *canvas;
+
+	CHECK_ARGS
+	SANITIZE_SIZE
+
 	shadow.x = x;
 	shadow.y = y;
 	shadow.width = width;
@@ -1868,8 +1948,6 @@ glide_draw_extension (GtkStyle * style,
 
 	pattern = glide_style->bg_solid[state_type];
 
-	gint gap_pos = 0, gap_shrink = 0;
-
 	if ((widget && GTK_IS_NOTEBOOK (widget)) && (state_type==GTK_STATE_NORMAL))
 	{ 
 		pattern = glide_style->active_tab_gradient[gap_side][state_type];
@@ -1947,7 +2025,7 @@ glide_draw_extension (GtkStyle * style,
 	/***********************************************/
 	/* GTK Special Cases - adjust Size/Offset      */
 	/***********************************************/
-	GlideSide side = GLIDE_SIDE_NONE;
+	side = GLIDE_SIDE_NONE;
 
 	switch (gap_side)
 	{
@@ -1974,8 +2052,6 @@ glide_draw_extension (GtkStyle * style,
 	/***********************************************/
 	/* Convert Shadow Type To Border Type          */
 	/***********************************************/
-	GlideBorderType border_type;
-
 	switch (shadow_type)
 	{
 		case GTK_SHADOW_ETCHED_IN:
@@ -2003,8 +2079,8 @@ glide_draw_extension (GtkStyle * style,
 	/***********************************************/
 	/* Draw Border                                 */
 	/***********************************************/
-	GlideBevelStyle bevel_style = GLIDE_BEVEL_STYLE_DEFAULT;
-	cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+	bevel_style = GLIDE_BEVEL_STYLE_DEFAULT;
+	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	glide_draw_pattern_fill(canvas, DEFAULT_BACKGROUND_PATTERN(glide_style, state_type, &pattern), shadow.x, shadow.y, shadow.width, shadow.height);
 
@@ -2042,6 +2118,7 @@ glide_draw_handle (GtkStyle * style,
              GtkOrientation orientation)
 {
   GlideStyle *glide_style = GLIDE_STYLE (style);
+  cairo_t *canvas;
 
   CHECK_ARGS
   SANITIZE_SIZE
@@ -2058,7 +2135,7 @@ glide_draw_handle (GtkStyle * style,
     area = NULL;
   }
 
-      cairo_t *canvas = ge_gdk_drawable_to_cairo (window, area);
+      canvas = ge_gdk_drawable_to_cairo (window, area);
  
       if (IS_HANDLE_BOX (widget))
 	{
@@ -2155,13 +2232,13 @@ glide_draw_handle (GtkStyle * style,
 	}
       else
 	{
+	  gboolean skip_shadow = FALSE;
+ 
 	do_glide_draw_grip (canvas,
 			&glide_style->color_cube.light[GTK_STATE_NORMAL],
 			&glide_style->color_cube.dark[GTK_STATE_NORMAL],
 			x, y, width, height, orientation == GTK_ORIENTATION_VERTICAL);
 
-	  gboolean skip_shadow = FALSE;
- 
           if (IS_BONOBO_DOCK_ITEM_GRIP(widget))
             {
               if IS_BOX(widget->parent)
@@ -2198,8 +2275,6 @@ glide_draw_handle (GtkStyle * style,
 	cairo_destroy(canvas);			          
 }
 
-extern GtkStyleClass *glide_parent_class;
-
 void
 glide_draw_focus(GtkStyle *style,
            GdkWindow *window,
@@ -2242,6 +2317,7 @@ glide_draw_focus(GtkStyle *style,
 	 	if (CHECK_DETAIL(detail, "button") && (ge_is_in_combo_box (widget)))
 		{
 			gint thick = 2;
+			GtkWidget *entry;
 
 			if ((!widget) || (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR))
 			{
@@ -2252,7 +2328,7 @@ glide_draw_focus(GtkStyle *style,
 				cairo_rectangle(canvas, x + thick, y + thick, width - line_width*2, height - thick*2);
 			}
 
-			GtkWidget *entry = g_object_get_data(G_OBJECT(ge_find_combo_box_widget_parent (widget)), "entry");
+			entry = g_object_get_data(G_OBJECT(ge_find_combo_box_widget_parent (widget)), "entry");
 
 			if (GTK_IS_WIDGET(entry) && !GTK_WIDGET_HAS_FOCUS(entry))
 			{
@@ -2357,12 +2433,14 @@ glide_draw_layout (GtkStyle        *style,
 	     int              y,
 	     PangoLayout      *layout)
 {
+	GdkGC *gc;
+
 	CHECK_ARGS
 	use_text &= !ge_is_combo_box(widget, FALSE);
 
 	#warning FIXME - gdk gdk gdk
 
-	GdkGC *gc = use_text ? style->text_gc[state_type] : style->fg_gc[state_type];
+	gc = use_text ? style->text_gc[state_type] : style->fg_gc[state_type];
 	
 	if (area) 
 	{
