@@ -4,7 +4,7 @@
 typedef struct _MistStyle MistStyle;
 typedef struct _MistStyleClass MistStyleClass;
 
-extern GType mist_type_style;
+extern GType mist_type_style G_GNUC_INTERNAL;
 
 #define MIST_TYPE_STYLE              mist_type_style
 #define MIST_STYLE(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), MIST_TYPE_STYLE, MistStyle))
